@@ -46,17 +46,32 @@ Function last_command (void);
 void free_bindings (Binding p);
 
 /* buffer.c --------------------------------------------------------------- */
-int calculate_the_region (Region * rp);
-void init_buffer (Buffer * bp);
-Buffer *create_buffer (const char *name);
+#define FIELD(ty, field)                                \
+  ty get_buffer_ ## field (const Buffer *bp);           \
+  void set_buffer_ ## field (Buffer *bp, ty field);
+#define FIELD_STR(field)                                \
+  FIELD(const char *, field)
+#include "buffer.h"
+#undef FIELD
+#undef FIELD_STR
+#define FIELD(ty, field)                                \
+  ty get_region_ ## field (const Region *cp);           \
+  void set_region_ ## field (Region *cp, ty field);
+#include "region.h"
+#undef FIELD
 void free_buffer (Buffer * bp);
 void free_buffers (void);
-void set_buffer_name (Buffer * bp, const char *name);
-void set_buffer_filename (Buffer * bp, const char *filename);
-Buffer *find_buffer (const char *name, int cflag);
-char *make_buffer_name (const char *filename);
+void init_buffer (Buffer * bp);
+Buffer * buffer_new (void);
+const char *get_buffer_filename_or_name (Buffer * bp);
+void set_buffer_names (Buffer * bp, const char *filename);
+Buffer * find_buffer (const char *name);
 void switch_to_buffer (Buffer * bp);
+Region * region_new (void);
 int warn_if_readonly_buffer (void);
+int calculate_the_region (Region * rp);
+bool delete_region (const Region * rp);
+bool in_region (size_t lineno, size_t x, Region * rp);
 void set_temporary_buffer (Buffer * bp);
 size_t calculate_buffer_size (Buffer * bp);
 int transient_mark_mode (void);
@@ -64,8 +79,20 @@ void activate_mark (void);
 void deactivate_mark (void);
 size_t tab_width (Buffer * bp);
 char *copy_text_block (size_t startn, size_t starto, size_t size);
+Buffer *create_scratch_buffer (void);
+void kill_buffer (Buffer * kill_bp);
+Completion *make_buffer_completion (void);
+bool check_modified_buffer (Buffer * bp);
 
 /* completion.c ----------------------------------------------------------- */
+#define FIELD(ty, field)                                        \
+  ty get_completion_ ## field (const Completion *cp);           \
+  void set_completion_ ## field (Completion *cp, ty field);
+#define FIELD_STR(field)                        \
+  FIELD(const char *, field)
+#include "completion.h"
+#undef FIELD
+#undef FIELD_STR
 int completion_strcmp (const void *p1, const void *p2);
 Completion *completion_new (int fileflag);
 void free_completion (Completion * cp);
@@ -108,17 +135,16 @@ extern char coding_eol_cr[3];
 extern char coding_eol_undecided[3];
 int exist_file (const char *filename);
 astr get_home_dir (void);
+astr agetcwd (void);
 bool expand_path (astr path);
 astr compact_path (astr path);
 bool find_file (const char *filename);
-void kill_buffer (Buffer * kill_bp);
 void zile_exit (int doabort);
 
 /* funcs.c ---------------------------------------------------------------- */
 void set_mark_interactive (void);
 le *universal_argument (int keytype, int xarg);
 void write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...);
-bool delete_region (const Region * r);
 
 /* getkey.c --------------------------------------------------------------- */
 void pushkey (size_t key);
@@ -146,6 +172,12 @@ astr keyvectostr (gl_list_t keys);
 void free_kill_ring (void);
 
 /* line.c ----------------------------------------------------------------- */
+#define FIELD(ty, field)                                \
+  ty get_line_ ## field (const Line *wp);               \
+  void set_line_ ## field (Line *wp, ty field);
+  FIELD(const char *, field)
+#include "line.h"
+#undef FIELD
 Line *line_new (void);
 void line_delete (Line *l);
 Line *line_insert (Line *l, astr i);
@@ -185,6 +217,12 @@ extern Buffer *cur_bp, *head_bp;
 extern int thisflag, lastflag, last_uniarg;
 
 /* marker.c --------------------------------------------------------------- */
+#define FIELD(ty, field)                                \
+  ty get_marker_ ## field (const Marker *cp);           \
+  void set_marker_ ## field (Marker *cp, ty field);
+#include "marker.h"
+#undef FIELD
+Marker * marker_new (void);
 void free_marker (Marker * marker);
 void move_marker (Marker * marker, Buffer * bp, Point pt);
 Marker *copy_marker (Marker * marker);
@@ -231,6 +269,7 @@ void recenter (Window * wp);
 
 /* search.c --------------------------------------------------------------- */
 void init_search (void);
+void free_search (void);
 
 /* term_minibuf.c --------------------------------------------------------- */
 void term_minibuf_write (const char *fmt);
@@ -276,7 +315,7 @@ void init_variables (void);
 void free_variables (void);
 char *minibuf_read_variable_name (char *fmt, ...);
 void set_variable (const char *var, const char *val);
-const char *get_variable_doc (Buffer * bp, const char *var, char **defval);
+const char *get_variable_doc (const char *var, char **defval);
 const char *get_variable_bp (Buffer * bp, const char *var);
 const char *get_variable (const char *var);
 long get_variable_number_bp (Buffer * bp, const char *var);
@@ -284,6 +323,12 @@ long get_variable_number (const char *var);
 bool get_variable_bool (const char *var);
 
 /* window.c --------------------------------------------------------------- */
+#define FIELD(ty, field)                                \
+  ty get_window_ ## field (const Window *wp);           \
+  void set_window_ ## field (Window *wp, ty field);
+  FIELD(const char *, field)
+#include "window.h"
+#undef FIELD
 void create_first_window (void);
 Window *find_window (const char *name);
 void free_windows (void);
