@@ -32,6 +32,23 @@
 #include "extern.h"
 
 
+/*
+ * Structure
+ */
+struct Line
+{
+#define FIELD(ty, name) ty name;
+#include "line.h"
+#undef FIELD
+};
+
+#define FIELD(ty, field)                        \
+  GETTER (Line, line, ty, field)                \
+  SETTER (Line, line, ty, field)
+
+#include "line.h"
+#undef FIELD
+
 /*
  * Circular doubly-linked lists
  */
@@ -50,38 +67,38 @@ line_new (void)
 
 /* Delete a list, freeing its nodes */
 void
-line_delete (Line *l)
+line_delete (Line *lp)
 {
-  while (l->next != l)
-    line_remove (l->next);
+  while (lp->next != lp)
+    line_remove (lp->next);
 
-  free (l);
+  free (lp);
 }
 
 /* Insert a line into list after the given point, returning the new line */
 Line *
-line_insert (Line *l, astr i)
+line_insert (Line *lp, astr i)
 {
   Line *n = XZALLOC (Line);
 
-  n->next = l->next;
-  n->prev = l;
+  n->next = lp->next;
+  n->prev = lp;
   n->text = i;
-  l->next = l->next->prev = n;
+  lp->next = lp->next->prev = n;
 
   return n;
 }
 
 /* Remove a line from a list, if not sole line in list */
 void
-line_remove (Line *l)
+line_remove (Line *lp)
 {
-  if (l->next || l->prev)
+  if (lp->next || lp->prev)
     {
-      astr as = l->text;
-      l->prev->next = l->next;
-      l->prev->next->prev = l->prev;
-      free (l);
+      astr as = lp->text;
+      lp->prev->next = lp->next;
+      lp->prev->next->prev = lp->prev;
+      free (lp);
       astr_delete (as);
     }
 }
@@ -101,22 +118,29 @@ line_remove (Line *l)
 static void
 adjust_markers (Line * newlp, Line * oldlp, size_t pointo, int dir, ptrdiff_t delta)
 {
-  Marker *pt = point_marker (), *m;
+  Marker *m_pt = point_marker (), *m;
 
   assert (dir >= -1 && dir <= 1);
 
-  for (m = cur_bp->markers; m != NULL; m = m->next)
-    if (m->pt.p == oldlp && (dir == -1 || m->pt.o > pointo))
+  for (m = get_buffer_markers (cur_bp); m != NULL; m = get_marker_next (m))
+    if (get_marker_pt (m).p == oldlp && (dir == -1 || get_marker_pt (m).o > pointo))
       {
-        m->pt.p = newlp;
-        m->pt.o += delta - (pointo * dir);
-        m->pt.n += dir;
+        Point pt = get_marker_pt (m);
+        pt.p = newlp;
+        pt.o += delta - (pointo * dir);
+        pt.n += dir;
+        set_marker_pt (m, pt);
+      }
+    else if (get_marker_pt (m).n > get_buffer_pt (cur_bp).n)
+      {
+        Point pt = get_marker_pt (m);
+        pt.n += dir;
+        set_marker_pt (m, pt);
       }
-    else if (m->pt.n > cur_bp->pt.n)
-      m->pt.n += dir;
 
-  cur_bp->pt = pt->pt; /* This marker has been updated to new position */
-  free_marker (pt);
+  /* This marker has been updated to new position. */
+  set_buffer_pt (cur_bp, get_marker_pt (m_pt));
+  free_marker (m_pt);
 }
 
 /* Insert the character at the current position and move the text at its right
@@ -129,9 +153,9 @@ intercalate_char (int c)
   if (warn_if_readonly_buffer ())
     return false;
 
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, 1);
-  astr_insert_char (cur_bp->pt.p->text, cur_bp->pt.o, c);
-  cur_bp->flags |= BFLAG_MODIFIED;
+  undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), 0, 1);
+  astr_insert_char (get_buffer_pt (cur_bp).p->text, get_buffer_pt (cur_bp).o, c);
+  set_buffer_modified (cur_bp, true);
 
   return true;
 }
@@ -148,44 +172,43 @@ insert_char (int c)
   if (warn_if_readonly_buffer ())
     return false;
 
-  if (cur_bp->flags & BFLAG_OVERWRITE)
+  if (get_buffer_overwrite (cur_bp))
     {
+      Point pt = get_buffer_pt (cur_bp);
       /* Current character isn't the end of line
          && isn't a \t
          || tab width isn't correct
          || current char is a \t && we are in the tab limit.  */
-      if ((cur_bp->pt.o < astr_len (cur_bp->pt.p->text))
+      if ((pt.o < astr_len (pt.p->text))
           &&
-          ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o) != '\t')
+          ((astr_get (pt.p->text, pt.o) != '\t')
            ||
-           ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o) ==
-             '\t') && ((get_goalc () % t) == t))))
+           ((astr_get (pt.p->text, pt.o) == '\t') && ((get_goalc () % t) == t))))
         {
           /* Replace the character.  */
           char ch = (char) c;
-          undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 1, 1);
-          astr_nreplace_cstr (cur_bp->pt.p->text, cur_bp->pt.o, 1, &ch, 1);
-          ++cur_bp->pt.o;
-
-          cur_bp->flags |= BFLAG_MODIFIED;
+          undo_save (UNDO_REPLACE_BLOCK, pt, 1, 1);
+          astr_nreplace_cstr (pt.p->text, pt.o, 1, &ch, 1);
+          ++pt.o;
+          set_buffer_pt (cur_bp, pt);
+          set_buffer_modified (cur_bp, true);
 
           return true;
         }
       /*
-       * Fall through the "insertion" mode of a character
-       * at the end of the line, since it is totally
-       * equivalent to "overwrite" mode.
+       * Fall through the "insertion" mode of a character at the end
+       * of the line, since it is the same as "overwrite" mode.
        */
     }
 
   intercalate_char (c);
   forward_char ();
-  adjust_markers (cur_bp->pt.p, cur_bp->pt.p, cur_bp->pt.o, 0, 1);
+  adjust_markers (get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).o, 0, 1);
 
   return true;
 }
 
-DEFUN_HIDDEN_ARGS ("insert-char", insert_char,
+DEFUN_NONINTERACTIVE_ARGS ("insert-char", insert_char,
                    STR_ARG (c))
 /*+
 Insert CHARACTER.
@@ -207,11 +230,12 @@ END_DEFUN
 int
 insert_char_in_insert_mode (int c)
 {
-  int old_overwrite = cur_bp->flags & BFLAG_OVERWRITE, ret;
+  int ret;
+  bool old_overwrite = get_buffer_overwrite (cur_bp);
 
-  cur_bp->flags &= ~BFLAG_OVERWRITE;
+  set_buffer_overwrite (cur_bp, false);
   ret = insert_char (c);
-  cur_bp->flags |= old_overwrite;
+  set_buffer_overwrite (cur_bp, old_overwrite);
 
   return ret;
 }
@@ -222,12 +246,12 @@ insert_expanded_tab (int (*inschr) (int chr))
   int c = get_goalc ();
   int t = tab_width (cur_bp);
 
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
   for (c = t - c % t; c > 0; --c)
     (*inschr) (' ');
 
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 
 static int
@@ -264,17 +288,17 @@ intercalate_newline (void)
   if (warn_if_readonly_buffer ())
     return false;
 
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, 1);
+  undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), 0, 1);
 
   /* Move the text after the point into a new line. */
-  line_insert (cur_bp->pt.p,
-               astr_substr (cur_bp->pt.p->text, cur_bp->pt.o,
-                            astr_len (cur_bp->pt.p->text) - cur_bp->pt.o));
-  ++cur_bp->last_line;
-  astr_truncate (cur_bp->pt.p->text, cur_bp->pt.o);
-  adjust_markers (cur_bp->pt.p->next, cur_bp->pt.p, cur_bp->pt.o, 1, 0);
+  line_insert (get_buffer_pt (cur_bp).p,
+               astr_substr (get_buffer_pt (cur_bp).p->text, get_buffer_pt (cur_bp).o,
+                            astr_len (get_buffer_pt (cur_bp).p->text) - get_buffer_pt (cur_bp).o));
+  set_buffer_last_line (cur_bp, get_buffer_last_line (cur_bp) + 1);
+  astr_truncate (get_buffer_pt (cur_bp).p->text, get_buffer_pt (cur_bp).o);
+  adjust_markers (get_buffer_pt (cur_bp).p->next, get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).o, 1, 0);
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  set_buffer_modified (cur_bp, true);
   thisflag |= FLAG_NEED_RESYNC;
 
   return true;
@@ -322,16 +346,16 @@ line_replace_text (Line ** lp, size_t offset, size_t oldlen,
 
   if (replace_case)
     {
-      int case_type = check_case (astr_char ((*lp)->text, offset),
-                                  oldlen);
+      case_type = check_case (astr_cstr ((*lp)->text) + offset, oldlen);
+
       if (case_type != 0)
         {
           as = astr_new_cstr (newtext);
-          astr_recase (as, case_type == 1 ? CAPITALIZED : UPPERCASE);
+          astr_recase (as, case_type == 1 ? case_capitalized : case_upper);
         }
     }
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  set_buffer_modified (cur_bp, true);
   astr_replace_cstr ((*lp)->text, offset, oldlen, newtext);
   adjust_markers (*lp, *lp, offset, 0, (ptrdiff_t) (newlen - oldlen));
 
@@ -350,67 +374,80 @@ line_replace_text (Line ** lp, size_t offset, size_t oldlen,
 bool
 fill_break_line (void)
 {
-  size_t i, break_col = 0, excess = 0, old_col;
+  size_t i, break_col = 0, old_col;
   size_t fillcol = get_variable_number ("fill-column");
+  bool break_made = false;
 
-  /* If we're not beyond fill-column, stop now. */
-  if (get_goalc () <= fillcol)
-    return false;
-
-  /* Move cursor back to fill column */
-  old_col = cur_bp->pt.o;
-  while (get_goalc () > fillcol + 1)
+  /* Only break if we're beyond fill-column. */
+  if (get_goalc () > fillcol)
     {
-      cur_bp->pt.o--;
-      excess++;
-    }
+      /* Save point. */
+      Marker *m = point_marker ();
 
-  /* Find break point moving left from fill-column. */
-  for (i = cur_bp->pt.o; i > 0; i--)
-    {
-      int c = *astr_char (cur_bp->pt.p->text, i - 1);
-      if (isspace (c))
+      /* Move cursor back to fill column */
+      old_col = get_buffer_pt (cur_bp).o;
+      while (get_goalc () > fillcol + 1)
         {
-          break_col = i;
-          break;
+          Point pt = get_buffer_pt (cur_bp);
+          pt.o--;
+          set_buffer_pt (cur_bp, pt);
         }
-    }
 
-  /* If no break point moving left from fill-column, find first
-     possible moving right. */
-  if (break_col == 0)
-    {
-      for (i = cur_bp->pt.o + 1; i < astr_len (cur_bp->pt.p->text); i++)
+      /* Find break point moving left from fill-column. */
+      for (i = get_buffer_pt (cur_bp).o; i > 0; i--)
         {
-          int c = *astr_char (cur_bp->pt.p->text, i - 1);
+          int c = astr_get (get_buffer_pt (cur_bp).p->text, i - 1);
           if (isspace (c))
             {
               break_col = i;
               break;
             }
         }
+
+      /* If no break point moving left from fill-column, find first
+         possible moving right. */
+      if (break_col == 0)
+        {
+          for (i = get_buffer_pt (cur_bp).o + 1;
+               i < astr_len (get_buffer_pt (cur_bp).p->text);
+               i++)
+            {
+              int c = astr_get (get_buffer_pt (cur_bp).p->text, i - 1);
+              if (isspace (c))
+                {
+                  break_col = i;
+                  break;
+                }
+            }
+        }
+
+      if (break_col >= 1) /* Break line. */
+        {
+          Point pt = get_buffer_pt (cur_bp);
+          pt.o = break_col;
+          set_buffer_pt (cur_bp, pt);
+          FUNCALL (delete_horizontal_space);
+          insert_newline ();
+          set_buffer_pt (cur_bp, get_marker_pt (m));
+          break_made = true;
+        }
+      else /* Undo fiddling with point. */
+        {
+          Point pt = get_buffer_pt (cur_bp);
+          pt.o = old_col;
+          set_buffer_pt (cur_bp, pt);
+        }
+
+      free_marker (m);
     }
 
-  if (break_col >= 1) /* Break line. */
-    {
-      size_t last_col = cur_bp->pt.o - break_col;
-      cur_bp->pt.o = break_col;
-      FUNCALL (delete_horizontal_space);
-      insert_newline ();
-      cur_bp->pt.o = last_col + excess;
-      return true;
-    }
-  else /* Undo fiddling with point. */
-    {
-      cur_bp->pt.o = old_col;
-      return false;
-    }
+  return break_made;
 }
 
 static int
 newline (void)
 {
-  if (cur_bp->flags & BFLAG_AUTOFILL &&
+  if (get_buffer_autofill (cur_bp) &&
       get_goalc () > (size_t) get_variable_number ("fill-column"))
     fill_break_line ();
   return insert_newline ();
@@ -439,7 +476,7 @@ void
 insert_nstring (const char *s, size_t len)
 {
   size_t i;
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, len);
+  undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), 0, len);
   undo_nosave = true;
   for (i = 0; i < len; i++)
     {
@@ -467,6 +504,7 @@ bprintf (const char *fmt, ...)
   xvasprintf (&buf, fmt, ap);
   va_end (ap);
   insert_nstring (buf, strlen (buf));
+  free (buf);
 }
 
 int
@@ -483,28 +521,28 @@ delete_char (void)
   if (warn_if_readonly_buffer ())
     return false;
 
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 1, 0);
+  undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), 1, 0);
 
   if (eolp ())
     {
-      size_t oldlen = astr_len (cur_bp->pt.p->text);
-      Line *oldlp = cur_bp->pt.p->next;
+      size_t oldlen = astr_len (get_buffer_pt (cur_bp).p->text);
+      Line *oldlp = get_buffer_pt (cur_bp).p->next;
 
       /* Join the lines. */
-      astr_cat (cur_bp->pt.p->text, oldlp->text);
+      astr_cat (get_buffer_pt (cur_bp).p->text, oldlp->text);
       line_remove (oldlp);
 
-      adjust_markers (cur_bp->pt.p, oldlp, oldlen, -1, 0);
-      --cur_bp->last_line;
+      adjust_markers (get_buffer_pt (cur_bp).p, oldlp, oldlen, -1, 0);
+      set_buffer_last_line (cur_bp, get_buffer_last_line (cur_bp) - 1);
       thisflag |= FLAG_NEED_RESYNC;
     }
   else
     {
-      astr_remove (cur_bp->pt.p->text, cur_bp->pt.o, 1);
-      adjust_markers (cur_bp->pt.p, cur_bp->pt.p, cur_bp->pt.o, 0, -1);
+      astr_remove (get_buffer_pt (cur_bp).p->text, get_buffer_pt (cur_bp).o, 1);
+      adjust_markers (get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).o, 0, -1);
     }
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  set_buffer_modified (cur_bp, true);
 
   return true;
 }
@@ -542,7 +580,7 @@ backward_delete_char_overwrite (void)
     insert_char (' ');
   backward_char ();
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  set_buffer_modified (cur_bp, true);
 
   return true;
 }
@@ -563,7 +601,7 @@ Delete the previous character.
 Join lines if the character is a newline.
 +*/
 {
-  int (*forward) (void) = cur_bp->flags & BFLAG_OVERWRITE ?
+  int (*forward) (void) = get_buffer_overwrite (cur_bp) ?
     backward_delete_char_overwrite : backward_delete_char;
   ok = execute_with_uniarg (true, uniarg, forward, delete_char);
 }
@@ -574,7 +612,7 @@ DEFUN ("delete-horizontal-space", delete_horizontal_space)
 Delete all spaces and tabs around point.
 +*/
 {
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
   while (!eolp () && isspace (following_char ()))
     delete_char ();
@@ -582,7 +620,7 @@ Delete all spaces and tabs around point.
   while (!bolp () && isspace (preceding_char ()))
     backward_delete_char ();
 
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 END_DEFUN
 
@@ -591,10 +629,10 @@ DEFUN ("just-one-space", just_one_space)
 Delete all spaces and tabs around point, leaving one space.
 +*/
 {
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   FUNCALL (delete_horizontal_space);
   insert_char_in_insert_mode (' ');
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 END_DEFUN
 
@@ -630,7 +668,6 @@ does nothing.
 +*/
 {
   size_t target_goalc = 0, cur_goalc = get_goalc ();
-  Marker *old_point;
   size_t t = tab_width (cur_bp);
 
   ok = leNIL;
@@ -641,11 +678,11 @@ does nothing.
   deactivate_mark ();
 
   /* If we're on first line, set target to 0. */
-  if (cur_bp->pt.p->prev == cur_bp->lines)
+  if (get_buffer_pt (cur_bp).p->prev == get_buffer_lines (cur_bp))
     target_goalc = 0;
   else
-    {				/* Find goalc in previous non-blank line. */
-      old_point = point_marker ();
+    { /* Find goalc in previous non-blank line. */
+      Marker *m = point_marker ();
 
       previous_nonblank_goalc ();
 
@@ -662,12 +699,12 @@ does nothing.
       if (!eolp ())
         target_goalc = get_goalc ();
 
-      cur_bp->pt = old_point->pt;
-      free_marker (old_point);
+      set_buffer_pt (cur_bp, get_marker_pt (m));
+      free_marker (m);
     }
 
   /* Insert indentation.  */
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   if (target_goalc > 0)
     {
       /* If not at EOL on target line, insert spaces & tabs up to
@@ -690,7 +727,7 @@ does nothing.
     }
   else
     ok = bool_to_lisp (insert_tab ());
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 END_DEFUN
 
@@ -698,7 +735,7 @@ static size_t
 previous_line_indent (void)
 {
   size_t cur_indent;
-  Marker *save_point = point_marker ();
+  Marker *m = point_marker ();
 
   FUNCALL (previous_line);
   FUNCALL (beginning_of_line);
@@ -710,8 +747,8 @@ previous_line_indent (void)
   cur_indent = get_goalc ();
 
   /* Restore point. */
-  cur_bp->pt = save_point->pt;
-  free_marker (save_point);
+  set_buffer_pt (cur_bp, get_marker_pt (m));
+  free_marker (m);
 
   return cur_indent;
 }
@@ -744,10 +781,10 @@ Indentation is done using the `indent-for-tab-command' function.
 
   deactivate_mark ();
 
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   if (insert_newline ())
     {
-      Marker *old_point = point_marker ();
+      Marker *m = point_marker ();
       int indent;
       size_t pos;
 
@@ -755,14 +792,14 @@ Indentation is done using the `indent-for-tab-command' function.
       previous_nonblank_goalc ();
       pos = get_goalc ();
       indent = pos > 0 || (!eolp () && isspace (following_char ()));
-      cur_bp->pt = old_point->pt;
-      free_marker (old_point);
+      set_buffer_pt (cur_bp, get_marker_pt (m));
+      free_marker (m);
       /* Only indent if we're in column > 0 or we're in column 0 and
          there is a space character there in the last non-blank line. */
       if (indent)
         FUNCALL (indent_for_tab_command);
       ok = leT;
     }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 END_DEFUN
