@@ -19,6 +19,8 @@
    Free Software Foundation, Fifth Floor, 51 Franklin Street, Boston,
    MA 02111-1301, USA.  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -54,7 +56,7 @@ static int
 read_char (astr as, size_t * pos)
 {
   if ((size_t) *pos < astr_len (as))
-    return *astr_char (as, (*pos)++);
+    return astr_get (as, (*pos)++);
   return EOF;
 }
 
