@@ -55,11 +55,11 @@ term_set_size (size_t cols, size_t rows)
 }
 
 static int
-make_char_printable (char **buf, size_t c)
+make_char_printable (char **buf, int c)
 {
   if (c == '\0')
     return xasprintf (buf, "^@");
-  else if (c <= '\32')
+  else if (c > 0 && c <= '\32')
     return xasprintf (buf, "^%c", 'A' + c - 1);
   else
     return xasprintf (buf, "\\%o", c & 0xff);
@@ -84,7 +84,7 @@ outch (int c, size_t font, size_t * x)
     term_addch (c), ++(*x);
   else
     {
-      j = make_char_printable (&buf, (size_t) c);
+      j = make_char_printable (&buf, c);
       for (w = 0; w < j && *x < term_width (); ++w)
         term_addch (buf[w]), ++(*x);
       free (buf);
@@ -93,37 +93,8 @@ outch (int c, size_t font, size_t * x)
   term_attrset (1, FONT_NORMAL);
 }
 
-static int
-in_region (size_t lineno, size_t x, Region * r)
-{
-  if (lineno >= r->start.n && lineno <= r->end.n)
-    {
-      if (r->start.n == r->end.n)
-        {
-          if (x >= r->start.o && x < r->end.o)
-            return true;
-        }
-      else if (lineno == r->start.n)
-        {
-          if (x >= r->start.o)
-            return true;
-        }
-      else if (lineno == r->end.n)
-        {
-          if (x < r->end.o)
-            return true;
-        }
-      else
-        {
-          return true;
-        }
-    }
-
-  return false;
-}
-
 static void
-draw_end_of_line (size_t line, Window * wp, size_t lineno, Region * r,
+draw_end_of_line (size_t line, Window * wp, size_t lineno, Region * rp,
                   int highlight, size_t x, size_t i)
 {
   if (x >= term_width ())
@@ -133,9 +104,9 @@ draw_end_of_line (size_t line, Window * wp, size_t lineno, Region * r,
     }
   else if (highlight)
     {
-      for (; x < wp->ewidth; ++i)
+      for (; x < get_window_ewidth (wp); ++i)
         {
-          if (in_region (lineno, i, r))
+          if (in_region (lineno, i, rp))
             outch (' ', FONT_REVERSE, &x);
           else
             x++;
@@ -150,79 +121,87 @@ draw_line (size_t line, size_t startcol, Window * wp, Line * lp,
   size_t x, i;
 
   term_move (line, 0);
-  for (x = 0, i = startcol; i < astr_len (lp->text) && x < wp->ewidth; i++)
+  for (x = 0, i = startcol; i < astr_len (get_line_text (lp)) && x < get_window_ewidth (wp); i++)
     {
       if (highlight && in_region (lineno, i, r))
-        outch (*astr_char (lp->text, i), FONT_REVERSE, &x);
+        outch (astr_get (get_line_text (lp), i), FONT_REVERSE, &x);
       else
-        outch (*astr_char (lp->text, i), FONT_NORMAL, &x);
+        outch (astr_get (get_line_text (lp), i), FONT_NORMAL, &x);
     }
 
   draw_end_of_line (line, wp, lineno, r, highlight, x, i);
 }
 
 static void
-calculate_highlight_region (Window * wp, Region * r, int *highlight)
+calculate_highlight_region (Window * wp, Region * rp, int *highlight)
 {
   if ((wp != cur_wp
        && !get_variable_bool ("highlight-nonselected-windows"))
-      || (!wp->bp->mark)
+      || (get_buffer_mark (get_window_bp (wp)) == NULL)
       || (!transient_mark_mode ())
-      || (transient_mark_mode () && !(wp->bp->flags & BFLAG_MARK)))
+      || (transient_mark_mode () && !get_buffer_mark_active (get_window_bp (wp))))
     {
       *highlight = false;
       return;
     }
 
   *highlight = true;
-  r->start = window_pt (wp);
-  r->end = wp->bp->mark->pt;
-  if (cmp_point (r->end, r->start) < 0)
-    swap_point (&r->end, &r->start);
+  set_region_start (rp, window_pt (wp));
+  set_region_end (rp, get_marker_pt (get_buffer_mark (get_window_bp (wp))));
+  if (cmp_point (get_region_end (rp), get_region_start (rp)) < 0)
+    {
+      Point pt1 = get_region_start (rp);
+      Point pt2 = get_region_end (rp);
+      swap_point (&pt1, &pt2);
+      set_region_start (rp, pt2);
+      set_region_end (rp, pt1);
+    }
 }
 
 static void
 draw_window (size_t topline, Window * wp)
 {
   size_t i, startcol, lineno;
-  Line *lp;
-  Region r;
+  Line * lp;
+  Region * rp = region_new ();
   int highlight;
   Point pt = window_pt (wp);
 
-  calculate_highlight_region (wp, &r, &highlight);
+  calculate_highlight_region (wp, rp, &highlight);
 
   /* Find the first line to display on the first screen line. */
-  for (lp = pt.p, lineno = pt.n, i = wp->topdelta;
-       i > 0 && lp->prev != wp->bp->lines;
-       lp = lp->prev, --i, --lineno)
+  for (lp = pt.p, lineno = pt.n, i = get_window_topdelta (wp);
+       i > 0 && get_line_prev (lp) != get_buffer_lines (get_window_bp (wp));
+       lp = get_line_prev (lp), --i, --lineno)
     ;
 
-  cur_tab_width = tab_width (wp->bp);
+  cur_tab_width = tab_width (get_window_bp (wp));
 
   /* Draw the window lines. */
-  for (i = topline; i < wp->eheight + topline; ++i, ++lineno)
+  for (i = topline; i < get_window_eheight (wp) + topline; ++i, ++lineno)
     {
       /* Clear the line. */
       term_move (i, 0);
       term_clrtoeol ();
 
       /* If at the end of the buffer, don't write any text. */
-      if (lp == wp->bp->lines)
+      if (lp == get_buffer_lines (get_window_bp (wp)))
         continue;
 
-      startcol = wp->start_column;
+      startcol = get_window_start_column (wp);
 
-      draw_line (i, startcol, wp, lp, lineno, &r, highlight);
+      draw_line (i, startcol, wp, lp, lineno, rp, highlight);
 
-      if (wp->start_column > 0)
+      if (get_window_start_column (wp) > 0)
         {
           term_move (i, 0);
           term_addch ('$');
         }
 
-      lp = lp->next;
+      lp = get_line_next (lp);
     }
+
+  free (rp);
 }
 
 static char *
@@ -230,12 +209,11 @@ make_mode_line_flags (Window * wp)
 {
   static char buf[3];
 
-  if ((wp->bp->flags & (BFLAG_MODIFIED | BFLAG_READONLY)) ==
-      (BFLAG_MODIFIED | BFLAG_READONLY))
+  if (get_buffer_modified (get_window_bp (wp)) && get_buffer_readonly (get_window_bp (wp)))
     buf[0] = '%', buf[1] = '*';
-  else if (wp->bp->flags & BFLAG_MODIFIED)
+  else if (get_buffer_modified (get_window_bp (wp)))
     buf[0] = buf[1] = '*';
-  else if (wp->bp->flags & BFLAG_READONLY)
+  else if (get_buffer_readonly (get_window_bp (wp)))
     buf[0] = buf[1] = '%';
   else
     buf[0] = buf[1] = '-';
@@ -251,36 +229,36 @@ make_mode_line_flags (Window * wp)
 static void
 calculate_start_column (Window * wp)
 {
-  size_t col = 0, lastcol = 0, t = tab_width (wp->bp);
+  size_t col = 0, lastcol = 0, t = tab_width (get_window_bp (wp));
   int rpfact, lpfact;
   char *buf;
-  const char *rp, *lp, *p;
+  size_t rp, lp, p;
   Point pt = window_pt (wp);
 
-  rp = astr_char (pt.p->text, pt.o);
-  rpfact = pt.o / (wp->ewidth / 3);
+  rp = pt.o;
+  rpfact = pt.o / (get_window_ewidth (wp) / 3);
 
-  for (lp = rp; lp >= astr_cstr (pt.p->text); --lp)
+  for (lp = rp; lp != SIZE_MAX; --lp)
     {
       for (col = 0, p = lp; p < rp; ++p)
-        if (*p == '\t')
+        if (astr_get (get_line_text (pt.p), p) == '\t')
           {
             col |= t - 1;
             ++col;
           }
-        else if (isprint ((int) *p))
+        else if (isprint ((int) astr_get (get_line_text (pt.p), p)))
           ++col;
         else
           {
-            col += make_char_printable (&buf, (size_t) * p);
+            col += make_char_printable (&buf, astr_get (get_line_text (pt.p), p));
             free (buf);
           }
 
-      lpfact = (lp - astr_cstr (pt.p->text)) / (wp->ewidth / 3);
+      lpfact = lp / (get_window_ewidth (wp) / 3);
 
-      if (col >= wp->ewidth - 1 || lpfact < (rpfact - 2))
+      if (col >= get_window_ewidth (wp) - 1 || lpfact < (rpfact - 2))
         {
-          wp->start_column = lp + 1 - astr_cstr (pt.p->text);
+          set_window_start_column (wp, lp + 1);
           point_screen_column = lastcol;
           return;
         }
@@ -288,7 +266,7 @@ calculate_start_column (Window * wp)
       lastcol = col;
     }
 
-  wp->start_column = 0;
+  set_window_start_column (wp, 0);
   point_screen_column = col;
 }
 
@@ -297,14 +275,15 @@ make_screen_pos (Window * wp, char **buf)
 {
   Point pt = window_pt (wp);
 
-  if (wp->bp->last_line <= wp->eheight && wp->topdelta == pt.n)
+  if (get_buffer_last_line (get_window_bp (wp)) <= get_window_eheight (wp) && get_window_topdelta (wp) == pt.n)
     xasprintf (buf, "All");
-  else if (pt.n == wp->topdelta)
+  else if (pt.n == get_window_topdelta (wp))
     xasprintf (buf, "Top");
-  else if (pt.n + (wp->eheight - wp->topdelta) > wp->bp->last_line)
+  else if (pt.n + (get_window_eheight (wp) - get_window_topdelta (wp)) > get_buffer_last_line (get_window_bp (wp)))
     xasprintf (buf, "Bot");
   else
-    xasprintf (buf, "%2d%%", (int) ((float) pt.n / wp->bp->last_line * 100));
+    xasprintf (buf, "%2d%%",
+               (int) ((float) pt.n / get_buffer_last_line (get_window_bp (wp)) * 100));
 
   return *buf;
 }
@@ -320,32 +299,32 @@ draw_status_line (size_t line, Window * wp)
   term_attrset (1, FONT_REVERSE);
 
   term_move (line, 0);
-  for (i = 0; i < wp->ewidth; ++i)
+  for (i = 0; i < get_window_ewidth (wp); ++i)
     term_addch ('-');
 
-  if (cur_bp->eol == coding_eol_cr)
+  if (get_buffer_eol (cur_bp) == coding_eol_cr)
     eol_type = "(Mac)";
-  else if (cur_bp->eol == coding_eol_crlf)
+  else if (get_buffer_eol (cur_bp) == coding_eol_crlf)
     eol_type = "(DOS)";
   else
     eol_type = ":";
 
   term_move (line, 0);
   bs = astr_afmt (astr_new (), "(%d,%d)", pt.n + 1,
-                  get_goalc_bp (wp->bp, window_pt (wp)));
+                  get_goalc_bp (get_window_bp (wp), window_pt (wp)));
   as = astr_afmt (astr_new (), "--%s%2s  %-15s   %s %-9s (Text",
-                  eol_type, make_mode_line_flags (wp), wp->bp->name,
+                  eol_type, make_mode_line_flags (wp), get_buffer_name (get_window_bp (wp)),
                   make_screen_pos (wp, &buf), astr_cstr (bs));
   free (buf);
   astr_delete (bs);
 
-  if (wp->bp->flags & BFLAG_AUTOFILL)
+  if (get_buffer_autofill (get_window_bp (wp)))
     astr_cat_cstr (as, " Fill");
-  if (wp->bp->flags & BFLAG_OVERWRITE)
+  if (get_buffer_overwrite (get_window_bp (wp)))
     astr_cat_cstr (as, " Ovwrt");
   if (thisflag & FLAG_DEFINING_MACRO)
     astr_cat_cstr (as, " Def");
-  if (wp->bp->flags & BFLAG_ISEARCH)
+  if (get_buffer_isearch (get_window_bp (wp)))
     astr_cat_cstr (as, " Isearch");
 
   astr_cat_char (as, ')');
@@ -365,7 +344,7 @@ term_redisplay (void)
 
   calculate_start_column (cur_wp);
 
-  for (wp = head_wp; wp != NULL; wp = wp->next)
+  for (wp = head_wp; wp != NULL; wp = get_window_next (wp))
     {
       if (wp == cur_wp)
         cur_topline = topline;
@@ -374,14 +353,14 @@ term_redisplay (void)
 
       /* Draw the status line only if there is available space after the
          buffer text space. */
-      if (wp->fheight - wp->eheight > 0)
-        draw_status_line (topline + wp->eheight, wp);
+      if (get_window_fheight (wp) - get_window_eheight (wp) > 0)
+        draw_status_line (topline + get_window_eheight (wp), wp);
 
-      topline += wp->fheight;
+      topline += get_window_fheight (wp);
     }
 
   /* Redraw cursor. */
-  term_move (cur_topline + cur_wp->topdelta, point_screen_column);
+  term_move (cur_topline + get_window_topdelta (cur_wp), point_screen_column);
 }
 
 void
